/*
 * Ben NanoNote communication with the Nu Electronics "Color LCD &
 * Joystick Shield" featuring a Nokia 6110 and the PCF8833 display
 * controller.
 *
 * http://shieldlist.org/nuelectronics/colorlcd-joystick
 *
 * Copyright (C) 2013 Paul Boddie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifndef __PCF8833_H__
#define __PCF8833_H__

#include <ubb/ubb.h>

/* Pin assignments:
 *
 * Sniffer  UBB   Shifters    Shield
 * -------  ----  ----------  ------
 * DAT2     DAT2
 * CD       DAT3  A2 (#1) B2      10 (SS/CS)
 * CMD      CMD   A1 (#1) B1       9 (RESET)
 * VCC      VDD   LV      HV     VIN
 * CLK      CLK   A4 (#1) B4      13 (SCLK/CLK)
 * GND      GND   GND    GND     GND
 * DAT0     DAT0  A3 (#1) B3      11 (MOSI/SEND)
 * DAT1     DAT1  A1 (#2) B1       8 (BACKLIGHT)
 */

#define LCD_CS          UBB_DAT3
#define LCD_RESET       UBB_CMD
#define LCD_CLK         UBB_CLK
#define LCD_SEND        UBB_DAT0
#define LCD_BACKLIGHT   UBB_DAT1

typedef enum {
    LCD_COMMAND = 0,
    LCD_DATA = 1
} lcd_sendmode;

/* Screen constants. */

#define SCREEN_WIDTH    132
#define SCREEN_X_MAX    131
#define SCREEN_HEIGHT   132
#define SCREEN_Y_MAX    131

/* Commands. */

#define LCD_NOP         0x00
#define LCD_SWRESET     0x01
#define LCD_SLEEPOUT    0x11
#define LCD_NORON       0x13
#define LCD_SETCON      0x25
#define LCD_DISPOFF     0x28
#define LCD_DISPON      0x29
#define LCD_CASET       0x2a
#define LCD_PASET       0x2b
#define LCD_RAMWR       0x2c
#define LCD_VSCRDEF     0x33
#define LCD_SEP         0x37
#define LCD_MADCTL      0x36
#define LCD_COLMOD      0x3a

/* Command-specific constants. */

#define LCD_MADCTL_MY_MX    0xc0
#define LCD_MADCTL_MY_V     0xa0
#define LCD_MADCTL_MY_MX_V  0xe0
#define LCD_MADCTL_MX_V     0x60

/* Functions. */

void spi_init();
void spi_begin();
void spi_end();
void spi_send(uint8_t v, lcd_sendmode mode);

void LCD_send(lcd_sendmode mode, uint8_t data);
void LCD_send_more_data(uint8_t data);
void LCD_send_raw(uint8_t data);
void LCD_init(void);
void LCD_window(uint8_t xmin, uint8_t ymin, uint8_t xmax, uint8_t ymax);
void LCD_blit_int(uint16_t colour);
void LCD_area(uint8_t xmin, uint8_t ymin, uint8_t xmax, uint8_t ymax, uint16_t colour);
void LCD_image(int x, int y, const uint16_t image[], uint8_t width, uint8_t height);
void LCD_image_region(int x, int y, const uint16_t image[], uint8_t width, uint8_t height, uint8_t from_x, uint8_t span_x, uint8_t from_y, uint8_t span_y);
void LCD_normal(void);
void LCD_scroll_area(uint8_t top_fixed, uint8_t scrolling, uint8_t bottom_fixed);
void LCD_scroll_start(uint8_t address);

#endif /* __PCF8833_H__ */

/* vim: tabstop=4 expandtab shiftwidth=4
 */
